<?php

namespace Controllers;

use Core\Language;
use Core\View;
use Core\Controller;
use Helpers\Auth;
use Helpers\Url;
use Models\Settings;
/*
 * Copyright 2014
 * ParaBlu Systems Private Limited
 * All Rights Reserved
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of ParaBlu Systems Private Limited and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary
 * to ParaBlu Systems Private Limited and its suppliers and may be covered by
 * Indian, US and Foreign Patents, patents in process, and are protected by
 * trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from ParaBlu Systems Private Limited.
 *
 * Login controller
 *
 * @author Prashanth Kumar B - prashanth.kumar@parablu.com
 * @version 2.0
 * @date June 10, 2015
 *
 */

class SettingsController extends Controller{

    /**
     * Below variable contains the object reference of a model
     */

    protected $_model;

    /**
     * Call the parent construct
     */
    public function __construct(){
        parent::__construct();
        $this->auth->loginCheck();
        $this->language->load('settings');
        $this->_model = new Settings();
        Auth::allowOnly('admin');
    }

    /**
     * Display for my account details
     */
    public function dashboard(){
        $data = array(
            'title' => $this->language->get('indexPageName'),
            'crumb1' => $this->language->get('dashboard'),
            'active' => 'settings',
            'csrf_token' => $this->setCSRFToken(),
            'loggedInUser' => $this->session->all()
        );

        View::renderTemplate('header', $data);
        View::renderTemplate('navigation', $data);
        View::render('settings/dashboard', $data);
        View::renderTemplate('footer', $data);
    }

    /**
     * Display for my account details
     */
    public function email(){
        $data = array(
            'title' => $this->language->get('indexPageName'),
            'crumb1' => $this->language->get('emailConfiguration'),
            'active' => 'settings',
            'csrf_token' => $this->setCSRFToken(),
            'loggedInUser' => $this->session->all()
        );

        View::renderTemplate('header', $data);
        View::renderTemplate('navigation', $data);
        View::render('settings/email', $data);
        View::renderTemplate('footer', $data);
    }

    public function writeToMailConfig(){
        if(!empty($_POST['hostName']) && !empty($_POST['portNumber']) && !empty($_POST['fromName']) && !empty($_POST['emailUsername']) && !empty($_POST['emailPassword'])){
            $values = htmlentities($_POST['hostName']).' '.htmlentities($_POST['portNumber']).' '.htmlentities($_POST['fromName']).' '.htmlentities($_POST['emailUsername']).' '.$_POST['emailPassword'];
            $scriptOutput = shell_exec('sh /var/lib/parablusys/bin/mailoverwrite.sh '.$values);
            $this->log('scriptOutput' . json_encode($scriptOutput), 'info');
            $data = array(
                'result'=>'success',
                'msg' => 'Details has been successfully saved'
            );
            echo json_encode($data);
        }else{
            $data = array(
                'result' => 'error',
                'msg'=>'None of the field listed should be empty'
            );
            echo json_encode($data);
        }
    }

    /**
     * Below Method Renders ODB Configuration View
     */
    public function connector(){

        $userDetails = $this->_model->getUserDetails(array('token' => $this->session->get('token')), $this->session->get("userName"));

        $headers = array(
            'Content-Type'  => 'application/json',
            'token'         => $this->session->get('token')
        );

        $result = $this->_model->getOdbDetails($headers);

        $data = array(
            'title' => $this->language->get('indexPageName'),
            'crumb1' => $this->language->get('connector'),
            'active' => 'settings',
            'csrf_token' => $this->setCSRFToken(),
            'userDetails' => $userDetails['body'],
            'loggedInUser' => $this->session->all(),

            'clientID'=> $result["body"][0]->{"clientId"},
            'clientRedirectURI'=> $result["body"][0]->{"clientRedirectURI"},
            'portalURL'=> $result["body"][0]->{"portalURL"},
            'clientSecretKey'=> $result["body"][0]->{"clientSecretKey"},
            'clientResourceURL'=> $result["body"][0]->{"clientResourceURL"},
            'configId'=> $result["body"][0]->{"configId"},

        );

        View::renderTemplate('header', $data);
        View::renderTemplate('navigation', $data);
        View::render('settings/connector', $data);
        View::renderTemplate('footer', $data);
    }

    public function writeODB(){
        if (!empty($_POST['clientId']) && !empty($_POST['clientSecret']) && !empty($_POST['clientRedirectURI']) && !empty($_POST['clientResourceURL']) && !empty($_POST['portalURL'])) {

            $headers = array(
                'Content-Type'  => 'application/json',
                'token'         => $this->session->get('token')
            );

            $odbDetails = array(
                'clientId' => $_POST['clientId'],
                'configId' => $_POST['configId'],
                'clientSecretKey' => $_POST['clientSecret'],
                'clientRedirectURI' => $_POST['clientRedirectURI'],
                'clientResourceURL' => $_POST['clientResourceURL'],
                'clientTokenURL' => $_POST['clientTokenURL'],
                'portalURL' => $_POST['portalURL']
            );
            $result = $this->_model->saveOdbDetails($headers, $odbDetails);
            $this->log(json_encode($odbDetails), "info");
            switch ($result['status']) {
                case (200):

                    $data = array(
                        'result' => 'success',
                        'msg' => 'Details has been successfully saved'
                    );
                    echo json_encode($data);
                    exit;

                case (421):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('421')
                    );
                    echo json_encode($result);
                    exit();
                case (500):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('500')
                    );
                    echo json_encode($result);
                    exit();
                case (0):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('0')
                    );
                    echo json_encode($result);
                    exit();
                default:
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('default')
                    );
                    echo json_encode($result);
                    exit();
            }
        } else {
            $data = array(
                "post" => $_POST,
                'result' => 'error',
                'msg' => 'None of the field listed should be empty'
            );
            echo json_encode($data);
        }

    }

    public function writeGoogle(){

if(!empty($_POST['clientId']) && !empty($_POST['clientSecret'])){
         $headers = array(
                'Content-Type'  => 'application/json',
                'token'         => $this->session->get('token')
            );

file_put_contents('/var/www/portal/app/Core/GoogleConfig.php',"<?php 
define('GOOGLE_CLIENT_ID','".$_POST['clientId']."');
define('GOOGLE_SECRET_KEY','".$_POST['clientSecret']."');
define('GOOGLE_REDIRECT_URI','http://accounts.google.com/o/oauth2/auth');
define('GOOGLE_TOKEN_URL', 'https://accounts.google.com/o/oauth2/token');
define('GOOGLE_USER_INFO_URL', 'https://www.googleapis.com/plus/v1/people/me?access_token=');
define('GOOGLE_SCOPE', 'email%20profile%20https://www.googleapis.com/auth/drive.file%20https://www.googleapis.com/auth/plus.me');
define('GOOGLE_RESOURCE_URI', 'googleOAuth');


?>");
            $googleDetails = array(
                'clientId' => $_POST['clientId'],
                'configId' => $_POST['configId'],
                'clientSecretKey' => $_POST['clientSecret']

            );
            $result = $this->_model->saveGoogleDetails($headers, $googleDetails);
            $this->log(json_encode($googleDetails), "info");
            switch ($result['status']) {
                case (200):
$data = array(
                        'result' => 'success',
                        'msg' => 'Details has been successfully saved'
                    );
                    echo json_encode($data);
                    exit;

                case (421):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('421')
                    );
                    echo json_encode($result);
                    exit();
                case (500):
$result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('500')
                    );
                    echo json_encode($result);
                    exit();
                case (0):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('0')
                    );
                    echo json_encode($result);
                    exit();
                default:
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('default')
                    );
                    echo json_encode($result);
                    exit();
  }
        }else {
            $data = array(
                "post" => $_POST,
                'result' => 'error',
                'msg' => 'None of the field listed should be empty'
            );
            echo json_encode($data);
        }
    }


    public function writeODC(){
        if(!empty($_POST['ClientId']) && !empty($_POST['clientSecret']) && !empty($_POST['clientRedirectURI']) && !empty($_POST['clientResourceURL']) && !empty($_POST['portalURL'])){
            $values = htmlentities($_POST['ClientId']).' '.htmlentities($_POST['clientSecret']).' '.htmlentities($_POST['clientRedirectURI']).' '.htmlentities($_POST['clientResourceURL']).' '.$_POST['portalURL'];
            $scriptOutput = shell_exec('sh /var/lib/parablusys/bin/odcoverwrite.sh '.$values);
            $this->log('scriptOutput' . json_encode($scriptOutput), 'info');
            $data = array(
                'result'=>'success',
                'msg' => 'Details has been successfully saved'
            );
            echo json_encode($data);
        }else{
            $data = array(
                'result' => 'error',
                'msg'=>'None of the field listed should be empty'
            );
            echo json_encode($data);
        }
    }

    /**
     * Below Method Renders proxy Configuration View
     */
    public function proxy(){
        $headers = array(
            'Content-Type'  => 'application/json',
            'token'         => $this->session->get('token')
        );
        $result = $this->_model->getProxyDetails($headers);
        $data = array(
            'title' => $this->language->get('indexPageName'),
            'crumb1' => $this->language->get('proxyConfiguration'),
            'active' => 'settings',
            'proxy' => $result['body']->{'proxyList'},
            'csrf_token' => $this->setCSRFToken(),
            'loggedInUser' => $this->session->all()
        );
        View::renderTemplate('header', $data);
        View::renderTemplate('navigation', $data);
        View::render('settings/proxy', $data);
        View::renderTemplate('footer', $data);
    }

    public function proxyremove(){
        $headers = array(
            'Content-Type'  => 'application/json',
            'token'         => $this->session->get('token')
        );
        $result = $this->_model->removeProxyDetails($headers,$_POST['proxyHost']);
        switch ($result['status']) {
            case (200):

                $response = array(
                    'result' => 'success',
                    'msg'    => $this->language->get('removeProxySettings')
                );
                echo json_encode($response);
                exit;

            case (421):
                $result = array(
                    'result' => 'fail',
                    'msg' => $this->language->get('421')
                );
                echo json_encode($result);
                exit();
            case (500):
                $result = array(
                    'result' => 'fail',
                    'msg' => $this->language->get('500')
                );
                echo json_encode($result);
                exit();
            case (0):
                $result = array(
                    'result' => 'fail',
                    'msg' => $this->language->get('0')
                );
                echo json_encode($result);
                exit();
            default:
                $result = array(
                    'result' => 'fail',
                    'msg' => $this->language->get('default')
                );
                echo json_encode($result);
                exit();
        }
    }

    /**
     *
     */
    public function writeProxySettings(){
        if(!empty($_POST['proxyURL']) && !empty($_POST['proxyUserName']) && !empty($_POST['proxyPassword']) && !empty($_POST['proxyPort'])){
            $headers = array(
                'Content-Type'  => 'application/json',
                'token'         => $this->session->get('token'),

            );
            $data = array(
                'host'     => $_POST['proxyURL'],
                'userName'      => $_POST['proxyUserName'],
                'password'      => $_POST['proxyPassword'],
                'port'          => $_POST['proxyPort']
            );
            $result = $this->_model->saveProxyDetails($headers,json_encode($data));
            $this->log("Result:".json_encode($result), "info");
            switch ($result['status']) {
                case (200):

                    $response = array(
                        'result' => 'success',
                        'msg'    => $this->language->get('Details has been successfully saved')
                    );
                    echo json_encode($response);
                    exit;

                case (421):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('421')
                    );
                    echo json_encode($result);
                    exit();
                case (500):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('500')
                    );
                    echo json_encode($result);
                    exit();
                case (0):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('0')
                    );
                    echo json_encode($result);
                    exit();
                default:
                    $result = array(
                        'result' => 'fail',
                        'response' => $result,
                        'msg' => $this->language->get('default')
                    );
                    echo json_encode($result);
                    exit();
            }
        }else{
            $data = array(
                'result' => 'error',
                'msg'=>'None of the field listed should be empty'
            );
            echo json_encode($data);
        }
    }
    
    public function editProxySettings(){
        if(!empty($_POST['proxyURL'])){
            $headers = array(
                'Content-Type'  => 'application/json',
                'token'         => $this->session->get('token'),

            );
            $data = array(
                'host'          => $_POST['proxyURL'],
                'userName'      => $_POST['proxyUserName'],
                'password'      => $_POST['proxyPassword'],
                'port'          => $_POST['proxyPort']
            );
            $proxyHost = $_POST['proxyURL'];
            $this->log("ResultEDIT:".json_encode($data), "info");
            $result = $this->_model->editProxyDetails($headers,json_encode($data),$proxyHost);
            $this->log("Result:".json_encode($result), "info");
            switch ($result['status']) {
                case (200):

                    $response = array(
                        'result' => 'success',
                        'msg'    => $this->language->get('Details has been successfully updated')
                    );
                    echo json_encode($response);
                    exit;

                case (421):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('421')
                    );
                    echo json_encode($result);
                    exit();
                case (500):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('500')
                    );
                    echo json_encode($result);
                    exit();
                case (0):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('0')
                    );
                    echo json_encode($result);
                    exit();
                default:
                    $result = array(
                        'result' => 'fail',
                        'response' => $result,
                        'msg' => $this->language->get('default')
                    );
                    echo json_encode($result);
                    exit();
            }
        }else{
            $data = array(
                'result' => 'error',
                'msg'=>'None of the field listed should be empty'
            );
            echo json_encode($data);
        }
    }
    

    /**
     * Below Method Renders Domain Host Name Configuration View
     */
    public function domain(){
        $data = array(
            'title' => $this->language->get('indexPageName'),
            'crumb1' => $this->language->get('domain'),
            'active' => 'settings',
            'csrf_token' => $this->setCSRFToken(),
            'loggedInUser' => $this->session->all()
        );

        View::renderTemplate('header', $data);
        View::renderTemplate('navigation', $data);
        View::render('settings/domain', $data);
        View::renderTemplate('footer', $data);
    }

    public function writeDomainHost(){
        if(!empty($_POST['domainName']) && !empty($_POST['hostName'])){
            $data = array(
                'result'=>'success',
                'msg' => 'Details has been successfully saved'
            );
            echo json_encode($data);
        }else{
            $data = array(
                'result' => 'error',
                'msg'=>'None of the field listed should be empty'
            );
            echo json_encode($data);
        }
    }

    /**
     * Below Method Renders SSL Certificate Upload View
     */
    public function sslAndEncryption(){
        $data = array(
            'title' => $this->language->get('indexPageName'),
            'crumb1' => $this->language->get('sslAndEncryption'),
            'active' => 'settings',
            'csrf_token' => $this->setCSRFToken(),
            'loggedInUser' => $this->session->all()
        );

        View::renderTemplate('header', $data);
        View::renderTemplate('navigation', $data);
        View::render('settings/sslAndEncryption', $data);
        View::renderTemplate('footer', $data);
    }

    public function saveSSLCertificate(){
        $tempFilePath = '/var/lib/parablusys/conf/';
        $flag = true;
        if(!empty($_FILES['mainCRTFile']) && !empty($_FILES['chainCRTFile']) && !empty($_FILES['keyFile'])){
            $tmp_name = $_FILES["mainCRTFile"]["tmp_name"];
            $name = $_FILES["mainCRTFile"]["name"];
            $ext1 = substr(strrchr($_FILES['mainCRTFile']['name'], "."), 1);
            if($ext1 != 'crt')
            {
                $flag = false;
            }
            move_uploaded_file($tmp_name, $tempFilePath.$name);
            $tmp_name = $_FILES["chainCRTFile"]["tmp_name"];
            $name = $_FILES["chainCRTFile"]["name"];
            $ext2 = substr(strrchr($_FILES['chainCRTFile']['name'], "."), 1);
            if($ext2 != 'crt')
            {
                $flag = false;
            }
            move_uploaded_file($tmp_name, $tempFilePath.$name);
            $tmp_name = $_FILES["keyFile"]["tmp_name"];
            $name = $_FILES["keyFile"]["name"];
            $ext3 = substr(strrchr($_FILES['keyFile']['name'], "."), 1);
            if($ext3 != 'key')
            {
                $flag = false;
            }
            move_uploaded_file($tmp_name, $tempFilePath.$name);
            $values = $_FILES["mainCRTFile"]["name"] . ' '.$_FILES["chainCRTFile"]["name"].' '.$_FILES["keyFile"]["name"].' '.$_POST["serverName"];
            $scriptOutput = shell_exec('sh /var/lib/parablusys/conf/cert_file.sh '.$values);
            $this->log('scriptOutput' . json_encode($scriptOutput), 'info');
            if($flag == 'true'){
            $data = array(
                'result'=>'success',
                'msg' => 'Details has been successfully saved'
            );
            }
            else {
             $data = array(
                'result'=>'fail',
                'msg' => 'file not supported, <br> check extension'
            );   
            }
            echo json_encode($data);
        }else{
            $data = array(
                'result' => 'error',
                'msg'=>'None of the field listed should be empty'
            );
            echo json_encode($data);
        }
    }

    /**
     * Below Method Renders Encryption settings page
     */
    public function encryption(){
        $keys = [];
        $headers = array(
            'Content-Type'      => 'application/json',
            'token'             => $this->session->get('token')
        );

        $result = $this->_model->getEncryption($headers);
        switch ($result['status']) {
            case (200):
                if(!empty($result['body']->{'encryptionKeyElementList'})){
                    foreach($result['body']->{'encryptionKeyElementList'} as $value){
                        date_default_timezone_set('Asia/Calcutta');


                        $a = $value->startTimestamp/1000;
                        //$date = new DateTime("@$a");
                        $value->startTime = date('d.m.Y H:i:s', $a);
                        $keys = array_merge(array($value), $keys);
                    }
                }
                $response = array(
                    'result' => 'success',
                    'keys' => $keys,
                );
                echo json_encode($response);
                exit;

            case (421):
                $result = array(
                    'result' => 'fail',
                    'msg' => $this->language->get('421')
                );
                echo json_encode($result);
                exit();
            case (500):
                $result = array(
                    'result' => 'fail',
                    'msg' => $this->language->get('500')
                );
                echo json_encode($result);
                exit();
            case (0):
                $result = array(
                    'result' => 'fail',
                    'msg' => $this->language->get('0')
                );
                echo json_encode($result);
                exit();
            default:
                $result = array(
                    'result' => 'fail',
                    'code' => $result,
                    'msg' => $this->language->get('default')
                );
                echo json_encode($result);
                exit();
        }
    }

    public function saveEncryption(){
        if(!empty($_POST)){
            $headers = array(
                'Content-Type'  => 'application/json',
                'token'         => $this->session->get('token')
            );

            $body = array(
                'userName'      => $this->session->get('userName'),
                'productType'   => $_POST['productType'],
                'saltKey'       => $_POST['saltKey']
            );
            $result = $this->_model->saveEncryption($headers,$body);
            switch ($result['status']) {
                case (200):

                    $response = array(
                        'result' => 'success',
                        'response' => $result,
                        'msg'    => $this->language->get('encryptionKeySaved')
                    );
                    echo json_encode($response);
                    exit;

                case (421):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('421')
                    );
                    echo json_encode($result);
                    exit();
                case (500):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('500')
                    );
                    echo json_encode($result);
                    exit();
                case (0):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('0')
                    );
                    echo json_encode($result);
                    exit();
                default:
                    $result = array(
                        'result' => 'fail',
                        'response' => $result,
                        'msg' => $this->language->get('default')
                    );
                    echo json_encode($result);
                    exit();
            }

        }else{
            echo json_encode(array(
                    'result' => 'noData',
                    'message' => $this->language->get('noData')
                )
            );
        }
    }

    /**
     * Below Method Renders Encryption settings page
     */
    public function local(){
        $headers = array(
            'Content-Type' => 'application/json',
            'token' => $this->session->get('token')
        );
        $result = $this->_model->getLocalStorageDetails($headers);
        $data = array(
            'title' => $this->language->get('indexPageName'),
            'crumb1' => $this->language->get('local'),
            'active' => 'settings',
            'csrf_token' => $this->setCSRFToken(),
            'localStorage' => $result['body'],
            'loggedInUser' => $this->session->all()
        );

        View::renderTemplate('header', $data);
        View::renderTemplate('navigation', $data);
        View::render('settings/local', $data);
        View::renderTemplate('footer', $data);
    }

    public function saveLocal(){
        if(!empty($_POST)){
            $headers = array(
                'Content-Type'      => 'application/json',
                'token'          => $this->session->get('token')
            );
            $data = array(
                'fileServerPath' => $_POST['path'],
                'fileServerUserName' => $_POST['userName'],
                'fileServerPassword' => $_POST['password']
            );
            $result = $this->_model->saveLocalStorageDetails($headers, $data);
            switch ($result['status']) {
                case (200):

                    $response = array(
                        'result' => 'success',
                        'msg'    => $this->language->get('encryptionKeySaved')
                    );
                    echo json_encode($response);
                    exit;

                case (421):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('421')
                    );
                    echo json_encode($result);
                    exit();
                case (500):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('500')
                    );
                    echo json_encode($result);
                    exit();
                case (0):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('0')
                    );
                    echo json_encode($result);
                    exit();
                default:
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('default')
                    );
                    echo json_encode($result);
                    exit();
            }

        }else{
            echo json_encode(array(
                    'result' => 'noData',
                    'message' => $this->language->get('noData')
                )
            );
        }
    }

    /**
     * Below Method Renders Encryption settings page
     */
    public function adCredentials(){

        $data = array(
            'title' => $this->language->get('indexPageName'),
            'crumb1' => $this->language->get('adCredentials'),
            'active' => 'settings',
            'csrf_token' => $this->setCSRFToken(),
            'loggedInUser' => $this->session->all()
        );

        View::renderTemplate('header', $data);
        View::renderTemplate('navigation', $data);
        View::render('settings/adCredentials', $data);
        View::renderTemplate('footer', $data);
    }

    public function getAdCredentials(){
        $adcS = [];
        $headers = array(
            'Content-Type'      => 'application/json',
            'token'          => $this->session->get('token')
        );
        $result = $this->_model->getAdCredentials($headers);

        if(!empty($result['body']->{'adElementList'})){
            foreach($result['body']->{'adElementList'} as $value){
                $adcS = array_merge(array($value), $adcS);
            }
        }

        switch ($result['status']) {
            case (200):

                $response = array(
                    'result' => 'success',
                    'adcS'    => $adcS
                );
                echo json_encode($response);
                exit;

            case (421):
                $result = array(
                    'result' => 'fail',
                    'msg' => $this->language->get('421')
                );
                echo json_encode($result);
                exit();
            case (500):
                $result = array(
                    'result' => 'fail',
                    'msg' => $this->language->get('500')
                );
                echo json_encode($result);
                exit();
            case (0):
                $result = array(
                    'result' => 'fail',
                    'msg' => $this->language->get('0')
                );
                echo json_encode($result);
                exit();
            default:
                $result = array(
                    'result' => 'fail',
                    'msg' => $this->language->get('default')
                );
                echo json_encode($result);
                exit();
        }
    }

    public function getAdCredential(){
        if(!empty($_POST['id'])){

            $headers = array(
                'Content-Type'      => 'application/json',
                'id'            => $_POST['id'],
                'token'             => $this->session->get('token')
            );

            $result = $this->_model->getAdCredential($headers);
            switch ($result['status']) {
                case (200):

                    $response = array(
                        'result' => 'success',
                        'adcS'    => $result['body']->{'adElement'}
                    );
                    echo json_encode($response);
                    exit;

                case (421):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('421')
                    );
                    echo json_encode($result);
                    exit();
                case (500):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('500')
                    );
                    echo json_encode($result);
                    exit();
                case (0):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('0')
                    );
                    echo json_encode($result);
                    exit();
                default:
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('default')
                    );
                    echo json_encode($result);
                    exit();
            }

        }else{
            echo json_encode(array(
                    'result' => 'noData',
                    'message' => $this->language->get('noData')
                )
            );
        }
    }

    public function saveAdCredentials(){
        if(!empty($_POST['userDn']) && !empty($_POST['adPassword']) && !empty($_POST['adCn']) && !empty($_POST['adUrl']) && !empty($_POST['adBase'])){
            $data = array(
                'userDn' => $_POST['userDn'],
                'adPassword' => $_POST['adPassword'],
                'adUrl' => $_POST['adUrl'],
                'adBase' => $_POST['adBase'],
                'adCn' => $_POST['adCn'],
                'adProductType' => $_POST['adProductType'],
                'adGroupName' => $_POST['adGroupName']
            );
            $this->log('Data: '. json_encode($data), 'info');
            $headers = array(
                'Content-Type'      => 'application/json',
                'token'          => $this->session->get('token')
            );
            $result = $this->_model->saveAdCredentials($headers, json_encode($data));
            $this->log('Result: '. json_encode($result), 'info');
            switch ($result['status']) {
                case (200):

                    $response = array(
                        'result' => 'success',
                        'msg'    => $this->language->get('savedAdCredentials')
                    );
                    echo json_encode($response);
                    exit;

                case (421):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('421')
                    );
                    echo json_encode($result);
                    exit();
                case (500):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('500')
                    );
                    echo json_encode($result);
                    exit();
                case (0):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('0')
                    );
                    echo json_encode($result);
                    exit();
                default:
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('default')
                    );
                    echo json_encode($result);
                    exit();
            }

        }else{
            echo json_encode(array(
                    'result' => 'noData',
                    'message' => $this->language->get('noData')
                )
            );
        }
    }
    
    public function editAdCredentials(){
        if(!empty($_POST['userDn']) && !empty($_POST['adPassword']) && !empty($_POST['adCn']) && !empty($_POST['adUrl']) && !empty($_POST['adBase'])){
            $data = array(
                'id' => $_POST['id'],
                'userDn' => $_POST['userDn'],
                'adPassword' => $_POST['adPassword'],
                'adUrl' => $_POST['adUrl'],
                'adBase' => $_POST['adBase'],
                'adCn' => $_POST['adCn'],
                'adProductType' => $_POST['adProductType'],
                'adGroupName' => $_POST['adGroupName']
            );
            $this->log('Data: '. json_encode($data), 'info');
            $headers = array(
                'Content-Type'      => 'application/json',
                'token'          => $this->session->get('token')
            );
            $result = $this->_model->editAdCredentials($headers, json_encode($data));
            $this->log('Result: '. json_encode($result), 'info');
            switch ($result['status']) {
                case (200):

                    $response = array(
                        'result' => 'success',
                        'msg'    => $this->language->get('savedAdCredentials')
                    );
                    echo json_encode($response);
                    exit;

                case (421):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('421')
                    );
                    echo json_encode($result);
                    exit();
                case (500):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('500')
                    );
                    echo json_encode($result);
                    exit();
                case (0):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('0')
                    );
                    echo json_encode($result);
                    exit();
                default:
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('default')
                    );
                    echo json_encode($result);
                    exit();
            }

        }else{
            echo json_encode(array(
                    'result' => 'noData',
                    'message' => $this->language->get('noData')
                )
            );
        }
    }

    public function removeAdCredentials(){
        if(!empty($_POST['userDn'])){
            $headers = array(
                'Content-Type'      => 'application/json',
                'userDn'            => $_POST['userDn'],
                'token'             => $this->session->get('token')
            );
            $result = $this->_model->removeAdCredentials($headers);
            switch ($result['status']) {
                case (200):

                    $response = array(
                        'result' => 'success',
                        'msg'    => $this->language->get('removeAdCredentials')
                    );
                    echo json_encode($response);
                    exit;

                case (421):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('421')
                    );
                    echo json_encode($result);
                    exit();
                case (500):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('500')
                    );
                    echo json_encode($result);
                    exit();
                case (0):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('0')
                    );
                    echo json_encode($result);
                    exit();
                default:
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('default')
                    );
                    echo json_encode($result);
                    exit();
            }

        }else{
            echo json_encode(array(
                    'result' => 'noData',
                    'message' => $this->language->get('noData')
                )
            );
        }
    }

    public function smtp(){

        $headers = array(
            'Content-Type'      => 'application/json',
            'token'             => $this->session->get('token')
        );

        $result = $this->_model->getSmtp($headers);

        $data = array(
            'title' => $this->language->get('indexPageName'),
            'crumb1' => $this->language->get('smtp'),
            'active' => 'settings',
            'csrf_token' => $this->setCSRFToken(),
            'loggedInUser' => $this->session->all(),

            "smtpHost" => !empty($result['body']->{'smtpElement'}->{'smtpHost'}) ? $result['body']->{'smtpElement'}->{'smtpHost'} : 'smtp.mail.yahoo.com',
            "sslPort" => !empty($result['body']->{'smtpElement'}->{'sslPort'}) ? $result['body']->{'smtpElement'}->{'sslPort'} : '465',
            "sslFactoryClass" => !empty($result['body']->{'smtpElement'}->{'sslFactoryClass'}) ? $result['body']->{'smtpElement'}->{'sslFactoryClass'} : "javax.net.ssl.SSLSocketFactory",
            "smtpPort" => !empty($result['body']->{'smtpElement'}->{'smtpPort'}) ? $result['body']->{'smtpElement'}->{'smtpPort'} : 465,
            "smtpAuth" => ($result['body']->{'smtpElement'}->{'smtpAuth'}) ? true : false,
            "smtpStarttlsEnable" => ($result['body']->{'smtpElement'}->{'smtpStarttlsEnable'}) ? true : false,
	    "smtpSSLEnable" => ($result['body']->{'smtpElement'}->{'smtpSSLEnable'}) ? true : false,
            "smtpUserName" => !empty($result['body']->{'smtpElement'}->{'smtpUserName'}) ? $result['body']->{'smtpElement'}->{'smtpUserName'} : 'noreply@parablu.com',
            "smtpPwd" => !empty($result['body']->{'smtpElement'}->{'smtpPwd'}) ? $result['body']->{'smtpElement'}->{'smtpPwd'} : 'P$an%1m*e@H4&b=?K:f',
            "fromEmail" => !empty($result['body']->{'smtpElement'}->{'fromEmail'}) ? $result['body']->{'smtpElement'}->{'fromEmail'} : 'noreply@parablu.com',
            "enableEmail" => ($result['body']->{'smtpElement'}->{'enableEmail'}) ? true : false
        );

        View::renderTemplate('header', $data);
        View::renderTemplate('navigation', $data);
        View::render('settings/smtp', $data);
        View::renderTemplate('footer', $data);
    }

    public function saveSmtp(){
        if(!empty($_POST)){
            $headers = array(
                'Content-Type'      => 'application/json',
                'token'             => $this->session->get('token')
            );
            $smtpAuth = 'false';    
                 if(isset($_POST['smtpAuth']) && $_POST['smtpAuth'] == 'true' ){
                     $smtpAuth = 'true';
                 }
                 $smtpStarttlsEnable = false;   
                 if(isset($_POST['smtpStarttlsEnable']) && $_POST['smtpStarttlsEnable'] == true ){
                     $smtpStarttlsEnable = true;
                 }
                 $smtpSSLEnable = false;    
                 if(isset($_POST['smtpSSLEnable']) && $_POST['smtpSSLEnable'] == true ){
                     $smtpSSLEnable = true;
                 }
                 $enableEmail = false;  
                 if(isset($_POST['enableEmail']) && $_POST['enableEmail'] == true ){
                     $enableEmail = true;
                 }

            $smtpDetails = array(
                "smtpHost" => $_POST["smtpHost"],
                "sslPort" => $_POST["sslPort"],
                "sslFactoryClass" => $_POST["factoryClass"],
                "smtpPort" => $_POST["smtpPort"],
                "smtpAuth" => $smtpAuth,
                "smtpStarttlsEnable" => $smtpStarttlsEnable,
        "smtpSSLEnable" => $smtpSSLEnable,
                "smtpUserName" => $_POST["smtpUname"],
                "smtpPwd" => $_POST["smtpPwd"],
                "fromEmail" => $_POST["email"],
                "enableEmail" => $enableEmail,
            );
            

            $result = $this->_model->saveSmtp($headers, $smtpDetails);

            switch ($result['status']) {
                case (200):

                    $response = array(
                        'result' => 'success',
                        'msg'    => $this->language->get('saveSmtp')
                    );
                    echo json_encode($response);
                    exit;

                case (421):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('421')
                    );
                    echo json_encode($result);
                    exit();
                case (500):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('500')
                    );
                    echo json_encode($result);
                    exit();
                case (0):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('0')
                    );
                    echo json_encode($result);
                    exit();
                default:
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('default')
                    );
                    echo json_encode($result);
                    exit();
            }

        }else{
            echo json_encode(array(
                    'result' => 'noData',
                    'message' => $this->language->get('noData')
                )
            );
        }
    }

    public function cloudProperties(){
        $headers = array(
            'Content-Type'      => 'application/json',
            'token'             => $this->session->get('token')
        );

        $result = $this->_model->getCloudProperties($headers);

        $data = array(
            'title' => $this->language->get('indexPageName'),
            'crumb1' => $this->language->get('cloudProperties'),
            'active' => 'settings',
            'csrf_token' => $this->setCSRFToken(),
            'loggedInUser' => $this->session->all(),

            "blusyncDownloadPath"       => !empty($result['body']->{'cloudPropertyElement'}->{'blusyncDownloadPath'}) ? $result['body']->{'cloudPropertyElement'}->{'blusyncDownloadPath'} : '/var/lib/parablusys/upgrade/',
            "mntParacloudPath"          => !empty($result['body']->{'cloudPropertyElement'}->{'mntParacloudPath'}) ? $result['body']->{'cloudPropertyElement'}->{'mntParacloudPath'} : '/mnt/',
            "cloudScriptsFolder"        => !empty($result['body']->{'cloudPropertyElement'}->{'cloudScriptsFolder'}) ? $result['body']->{'cloudPropertyElement'}->{'cloudScriptsFolder'} : '/var/lib/parablusys/bin/',
            "scriptsBinDir"             => !empty($result['body']->{'cloudPropertyElement'}->{'scriptsBinDir'}) ? $result['body']->{'cloudPropertyElement'}->{'scriptsBinDir'} : '/var/lib/parablusys/bin/',
            "scriptsVideoThumbnail"     => !empty($result['body']->{'cloudPropertyElement'}->{'scriptsVideoThumbnail'}) ? $result['body']->{'cloudPropertyElement'}->{'scriptsVideoThumbnail'} : 'generateVideoThumbnails.sh',
            "scriptsGeneralThumbnail"   => !empty($result['body']->{'cloudPropertyElement'}->{'scriptsGeneralThumbnail'}) ? $result['body']->{'cloudPropertyElement'}->{'scriptsGeneralThumbnail'} : 'generateThumbnails.sh',
            "cloudFileName"             => !empty($result['body']->{'cloudPropertyElement'}->{'cloudFileName'}) ? $result['body']->{'cloudPropertyElement'}->{'cloudFileName'} : 'cloudName',
            "cloudType"                 => !empty($result['body']->{'cloudPropertyElement'}->{'cloudType'}) ? $result['body']->{'cloudPropertyElement'}->{'cloudType'} : 'enterprise',
            "solrUrl"                   => !empty($result['body']->{'cloudPropertyElement'}->{'solrUrl'}) ? $result['body']->{'cloudPropertyElement'}->{'solrUrl'} : 'http://192.168.7.57/ParacloudSearch/search/',
            "supportMail"               => !empty($result['body']->{'cloudPropertyElement'}->{'supportMail'}) ? $result['body']->{'cloudPropertyElement'}->{'supportMail'} : 'praveen.anandaraju@parablu.com',
            "sslAuthEnabled"            => ($result['body']->{'cloudPropertyElement'}->{'sslAuthEnabled'}) ? true : false,
            "azureVM"                   => ($result['body']->{'cloudPropertyElement'}->{'azureVM'}) ? true : false,
            "localAD"                   => ($result['body']->{'cloudPropertyElement'}->{'localAD'}) ? true : false,
            "reportRows"                => !empty($result['body']->{'cloudPropertyElement'}->{'reportRows'}) ? $result['body']->{'cloudPropertyElement'}->{'reportRows'} : '50',
            "emailToAdminOnly"          => ($result['body']->{'cloudPropertyElement'}->{'emailToAdminOnly'}) ? true : false,
            "maxCrawlHit"               => !empty($result['body']->{'cloudPropertyElement'}->{'maxCrawlHit'}) ? $result['body']->{'cloudPropertyElement'}->{'maxCrawlHit'} : '100',
            "defaultBkpPolicyName"      => !empty($result['body']->{'cloudPropertyElement'}->{'defaultBkpPolicyName'}) ? $result['body']->{'cloudPropertyElement'}->{'defaultBkpPolicyName'} : 'STATISTICS_COLLECTOR',
            "defaultSyncPolicyName"     => !empty($result['body']->{'cloudPropertyElement'}->{'defaultSyncPolicyName'}) ? $result['body']->{'cloudPropertyElement'}->{'defaultSyncPolicyName'} : 'STATISTICS_SYNC_COLLECTOR',
            "mongoReadPreference"       => ($result['body']->{'cloudPropertyElement'}->{'mongoReadPreference'}) ? true : false,
            "writeConcernLimit"         => !empty($result['body']->{'cloudPropertyElement'}->{'writeConcernLimit'}) ? $result['body']->{'cloudPropertyElement'}->{'writeConcernLimit'} : 1000,
            "ebmsWriteDelayLimit"       => !empty($result['body']->{'cloudPropertyElement'}->{'ebmsWriteDelayLimit'}) ? $result['body']->{'cloudPropertyElement'}->{'ebmsWriteDelayLimit'} : 10,
            "noOfMultiADConfig"         => !empty($result['body']->{'cloudPropertyElement'}->{'noOfMultiADConfig'}) ? $result['body']->{'cloudPropertyElement'}->{'noOfMultiADConfig'} : 1,
            "isCrawlEnabled"            => ($result['body']->{'cloudPropertyElement'}->{'isCrawlEnabled'}) ? true : false,
            "emailBkpGroupName"         => !empty($result['body']->{'cloudPropertyElement'}->{'emailBkpGroupName'}) ? $result['body']->{'cloudPropertyElement'}->{'emailBkpGroupName'} : '',
            "emailSyncGroupName"        => !empty($result['body']->{'cloudPropertyElement'}->{'emailSyncGroupName'}) ? $result['body']->{'cloudPropertyElement'}->{'emailSyncGroupName'} : '',
            "bkpGroupEmail"             => !empty($result['body']->{'cloudPropertyElement'}->{'bkpGroupEmail'}) ? $result['body']->{'cloudPropertyElement'}->{'bkpGroupEmail'} : 'MySecureBackup',
            "syncGroupEmail"            => !empty($result['body']->{'cloudPropertyElement'}->{'syncGroupEmail'}) ? $result['body']->{'cloudPropertyElement'}->{'syncGroupEmail'} : 'MySecureSync',
            "chunkUpdateMaxRetry"       => !empty($result['body']->{'cloudPropertyElement'}->{'chunkUpdateMaxRetry'}) ? $result['body']->{'cloudPropertyElement'}->{'chunkUpdateMaxRetry'} : 3,
            "isDomainCheckEnabled"      => ($result['body']->{'cloudPropertyElement'}->{'isDomainCheckEnabled'}) ? true : false,
            "canAdminRestoreAll"        => ($result['body']->{'cloudPropertyElement'}->{'canAdminRestoreAll'}) ? true : false,
            "clientMailName"            => !empty($result['body']->{'cloudPropertyElement'}->{'clientMailName'}) ? $result['body']->{'cloudPropertyElement'}->{'clientMailName'} : 'Parablu',
            "clientSupportMailName"     => !empty($result['body']->{'cloudPropertyElement'}->{'clientSupportMailName'}) ? $result['body']->{'cloudPropertyElement'}->{'clientSupportMailName'} : 'support@parablu.com',
            "isShareFileNameEnabled"    => ($result['body']->{'cloudPropertyElement'}->{'isShareFileNameEnabled'}) ? true : false,
            "adfsUrl"                   => !empty($result['body']->{'cloudPropertyElement'}->{'adfsUrl'}) ? $result['body']->{'cloudPropertyElement'}->{'adfsUrl'} : 'https:443//testv20-1.parablu.com/springsam/saml/login',
            "adGroupName"               => !empty($result['body']->{'cloudPropertyElement'}->{'adGroupName'}) ? $result['body']->{'cloudPropertyElement'}->{'adGroupName'} : '',
	    "bulkEmailDelay"            => ($result['body']->{'cloudPropertyElement'}->{'bulkEmailDelay'}) ? $result['body']->{'cloudPropertyElement'}->{'bulkEmailDelay'} : "NO DELAY",
	    "notifyForRestores"         => ($result['body']->{'cloudPropertyElement'}->{'isEmailNotifierForRestoreEnabled'}) ? true : false,
            "notifyForBackup"           => ($result['body']->{'cloudPropertyElement'}->{'isEmailNotifierEnabledForFirstBackupCompletion'}) ? true : false,
        );

        View::renderTemplate('header', $data);
        View::renderTemplate('navigation', $data);
        View::render('settings/cloudProperties', $data);
        View::renderTemplate('footer', $data);
    }

    public function saveCloudProperties(){
        if(!empty($_POST)){
            $headers = array(
                'Content-Type'      => 'application/json',
                'token'             => $this->session->get('token')
            );

	    if(empty($_POST['bulkEmailDelay']) || $_POST['bulkEmailDelay'] == 'NO DELAY'){
                $_POST['bulkEmailDelay'] = 0;
            }

            $cloudDetails = array(
                "blusyncDownloadPath" => $_POST["blusyncDownloadPath"],
                "mntParacloudPath" => $_POST["mntParacloudPath"],
                "cloudScriptsFolder" => $_POST["cloudScriptsFolder"],
                "scriptsBinDir" => $_POST["scriptsBinDir"],
                "scriptsVideoThumbnail" => $_POST["scriptsVideoThumbnail"],
                "scriptsGeneralThumbnail" => $_POST["scriptsGeneralThumbnail"],
                "cloudFileName" => $_POST["cloudFileName"],
                "cloudType" => $_POST["cloudType"],
                "solrUrl" => $_POST["solrUrl"],
                "supportMail" => $_POST["supportMail"],
                "sslAuthEnabled" => $_POST["sslAuthEnabled"],
                "azureVM" => $_POST["azureVM"],
                "localAD" => $_POST["localAD"],
                "reportRows" => $_POST["reportRows"],
                "emailToAdminOnly" => $_POST["emailToAdminOnly"],
                "maxCrawlHit" => $_POST["maxCrawlHit"],
                "defaultBkpPolicyName" => $_POST["defaultBkpPolicyName"],
                "defaultSyncPolicyName" => $_POST["defaultSyncPolicyName"],
                "mongoReadPreference" => $_POST["mongoReadPreference"],
                "writeConcernLimit" => $_POST["writeConcernLimit"],
                "ebmsWriteDelayLimit" => $_POST["ebmsWriteDelayLimit"],
                "noOfMultiADConfig" => $_POST["noOfMultiADConfig"],
                "isCrawlEnabled" => $_POST["isCrawlEnabled"],
                "emailBkpGroupName" => $_POST["emailBkpGroupName"],
                "emailSyncGroupName" => $_POST["emailSyncGroupName"],
                "bkpGroupEmail" => $_POST["bkpGroupEmail"],
                "syncGroupEmail" => $_POST["syncGroupEmail"],
                "chunkUpdateMaxRetry" => $_POST["chunkUpdateMaxRetry"],
                "isDomainCheckEnabled" => $_POST["isDomainCheckEnabled"],
                "canAdminRestoreAll" => $_POST["canAdminRestoreAll"],
                "clientMailName" => $_POST["clientMailName"],
                "clientSupportMailName" => $_POST["clientSupportMailName"],
                "isShareFileNameEnabled" => $_POST["isShareFileNameEnabled"],
                "adfsUrl" => $_POST["adfsUrl"],
                "adGroupName" => $_POST["adGroupName"],
		"bulkEmailDelay" => $_POST["bulkEmailDelay"],
		"isEmailNotifierForRestoreEnabled" => $_POST["notifyForRestores"],
                "isEmailNotifierEnabledForFirstBackupCompletion" => $_POST["notifyForBackup"],
            );

            $result = $this->_model->saveCloudProperties($headers, $cloudDetails);
            $this->log(json_encode($cloudDetails), "info");
            switch ($result['status']) {
                case (200):

                    $response = array(
                        'result' => 'success',
                        'msg'    => $this->language->get('savedCloudProperties')
                    );
                    echo json_encode($response);
                    exit;

                case (421):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('421')
                    );
                    echo json_encode($result);
                    exit();
                case (500):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('500')
                    );
                    echo json_encode($result);
                    exit();
                case (0):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('0')
                    );
                    echo json_encode($result);
                    exit();
                default:
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('default')
                    );
                    echo json_encode($result);
                    exit();
            }

        }else{
            echo json_encode(array(
                    'result' => 'noData',
                    'message' => $this->language->get('noData')
                )
            );
        }
    }

    /**
     * Below Method Renders Encryption settings page
     */
    public function cloudCredentials(){
        $userDetails = $this->_model->getUserDetails(array('token' => $this->session->get('token')), $this->session->get("userName"));
        $headers = array(
            'Content-Type'  => 'application/json',
            'token'         => $this->session->get('token')
        );

        $result = $this->_model->getOdbDetails($headers);
        $data = array(
            'title' => $this->language->get('indexPageName'),
            'crumb1' => $this->language->get('cloudCredentials'),
            'active' => 'settings',
            'userDetails' => $userDetails['body'],
            'loggedInUser' => $this->session->all(),
            'csrf_token' => $this->setCSRFToken(),

            'cloudCredentialName' => $result["body"]->{"cloudCredentialName"},
            'clientID'=> $result["body"]->{"clientId"},
            'clientRedirectURI'=> $result["body"]->{"clientRedirectURI"},
            'portalURL'=> $result["body"]->{"portalURL"},
            'clientSecretKey'=> $result["body"]->{"clientSecretKey"},
            'clientResourceURL'=> $result["body"]->{"clientResourceURL"},
            'clientTokenURL'=> $result["body"]->{"clientTokenURL"},
            'configId'=> $result["body"]->{"configId"},
        );

        View::renderTemplate('header', $data);
        View::renderTemplate('navigation', $data);
        View::render('settings/cloudCredentials', $data);
        View::renderTemplate('footer', $data);
    }

    public function getAllCcS(){
        if(empty($_POST)){
            echo json_encode(array(
                    'result' => 'noData',
                    'message' => $this->language->get('noData')
                )
            );
        }
        $ccS = [];
        $headers = array(
            'Content-Type'      => 'application/json',
            'token'          => $this->session->get('token')
        );
        $result = $this->_model->getCloudCredentials($headers);
        $this->log($result);
        switch ($result['status']) {
            case (200):
                if(!empty($result['body']->{'cloudCredentialListElement'}->{'cloudCredentialElements'})){
                    foreach($result['body']->{'cloudCredentialListElement'}->{'cloudCredentialElements'} as $value){
                        switch($value->cloudName){
                            case 'azureBlob':
                                $value->cloudNameInCaps = Language::show('azureBlob', 'settings');
                                break;
                            case 'ibmCloud':
                                $value->cloudNameInCaps = Language::show('ibmSoftlayer', 'settings');
                                break;
                            case 'oneDrive':
                                $value->cloudNameInCaps = Language::show('microsoft', 'settings');
                                break;
                            case 'googleDrive':
                                $value->cloudNameInCaps = Language::show('googleDrive', 'settings');
                                break;
                            case 'amazon':
                                $value->cloudNameInCaps = Language::show('amazonS3', 'settings');
                                break;
                            case 'localStorage':
                                $value->cloudNameInCaps = Language::show('localStorage', 'settings');
                                break;
                            default:
                                $value->cloudNameInCaps = '';
                                break;
                        }

                        $ccS = array_merge(array($value), $ccS);
                    }
                }
                $response = array(
                    'result' => 'success',
                    'ccS' => array_reverse($ccS),
                );
                echo json_encode($response);
                exit;

            case (421):
                $result = array(
                    'result' => 'fail',
                    'msg' => $this->language->get('421')
                );
                echo json_encode($result);
                exit();
            case (500):
                $result = array(
                    'result' => 'fail',
                    'msg' => $this->language->get('500')
                );
                echo json_encode($result);
                exit();
            case (0):
                $result = array(
                    'result' => 'fail',
                    'msg' => $this->language->get('0')
                );
                echo json_encode($result);
                exit();
            default:
                $result = array(
                    'result' => 'fail',
                    'code' => $result,
                    'msg' => $this->language->get('default')
                );
                echo json_encode($result);
                exit();
        }
    }
    
    public function getCloudCredential(){

            if(empty($_POST)){
                echo json_encode(array(
                        'result' => 'noData',
                        'message' => $this->language->get('noData')
                    )
                );
            }
            $ccS1 = [];
            $headers = array(
                'Content-Type'      => 'application/json',
                'identity'            => $_POST['identity'],
                'token'             => $this->session->get('token')
            );

        $result = $this->_model->getCloudCredentials($headers);
        $this->log($result);
        switch ($result['status']) {
            case (200):
                if(!empty($result['body']->{'cloudCredentialListElement'}->{'cloudCredentialElements'})){
                    foreach($result['body']->{'cloudCredentialListElement'}->{'cloudCredentialElements'} as $value){
                        if($value->identity == $_POST['identity']){
                            switch($value->cloudName){
                            case 'azureBlob':
                                $value->cloudNameInCaps = Language::show('azureBlob', 'settings');
                                break;
                            case 'ibmCloud':
                                $value->cloudNameInCaps = Language::show('ibmSoftlayer', 'settings');
                                break;
                            case 'oneDrive':
                                $value->cloudNameInCaps = Language::show('microsoft', 'settings');
                                break;
                            case 'googleDrive':
                                $value->cloudNameInCaps = Language::show('googleDrive', 'settings');
                                break;
                            case 'amazon':
                                $value->cloudNameInCaps = Language::show('amazonS3', 'settings');
                                break;
                            case 'localStorage':
                                $value->cloudNameInCaps = Language::show('localStorage', 'settings');
                                break;
                            default:
                                $value->cloudNameInCaps = '';
                                break;
                        }
                            $ccS1 = array_merge(array($value), $ccS1);
                            $this->log('Resultfffff: '. json_encode($ccS1), 'info');
                        }


                    }
                }
                $response = array(
                    'result' => 'success',
                    'ccS1' => $ccS1,
                );
                echo json_encode($response);
                exit;

            case (421):
                $result = array(
                    'result' => 'fail',
                    'msg' => $this->language->get('421')
                );
                echo json_encode($result);
                exit();
            case (500):
                $result = array(
                    'result' => 'fail',
                    'msg' => $this->language->get('500')
                );
                echo json_encode($result);
                exit();
            case (0):
                $result = array(
                    'result' => 'fail',
                    'msg' => $this->language->get('0')
                );
                echo json_encode($result);
                exit();
            default:
                $result = array(
                    'result' => 'fail',
                    'code' => $result,
                    'msg' => $this->language->get('default')
                );
                echo json_encode($result);
                exit();
        }

    }
    public function saveCloudCredentials(){
        if(!empty($_POST['cloudType'])){
	if(empty($_POST['defaultCloudCredentials'])){
            $_POST['defaultCloudCredentials'] = false;
            }else{
            $_POST['defaultCloudCredentials'] = true;
            }
            /*$data = array(
                'cloudName' => $_POST['cloudType'],
                'identity' => ($_POST['cloudType'] == "localStorage") ? "local" : $_POST['identity'],
                'credential' => $_POST['accessKey'],
                'containerName' => !empty($_POST['containerName']) ? $_POST['containerName'] : '',
                'endPointUrl' => ($_POST['cloudType'] == "localStorage") ? $_POST['localStorage'] : $_POST['epaUrl'],
            );*/
            if($_POST['cloudType'] == "azureBlob" || $_POST['cloudType'] == "ibmCloud" || $_POST['cloudType'] == "amazon" || $_POST['cloudType'] == "localStorage"){


                $data = array(

                    'cloudName' => $_POST['cloudType'],
                    'cloudCredentialName' => $_POST["cloudCredentialName"],
                    'identity' => ($_POST['cloudType'] == "localStorage") ? "local" : $_POST['identity'],
                    'provider' => $_POST['cloudType'],
                    'credential' => $_POST['accessKey'],
                    'containerName' => !empty($_POST['containerName']) ? $_POST['containerName'] : '',
                    'endPointUrl' => ($_POST['cloudType'] == "localStorage") ? $_POST['localStorage'] : $_POST['epaUrl'],
		    'defaultCloudCredentials' => $_POST['defaultCloudCredentials']
                );

            }
            elseif($_POST['cloudType'] == "googleDrive" || $_POST['cloudType'] == "oneDrive"){
                $data = array(
                    'cloudName' => $_POST['cloudType'],
                    'cloudCredentialName' => $_POST["cloudCredentialName"],
                    'provider' => $_POST['cloudType'],
                    'identity' => $_POST['clientId'],
                    'credential' => $_POST['clientSecret'],
                    'containerName' => !empty($_POST['containerName']) ? $_POST['containerName'] : '',
                    'endPointUrl' => !empty($_POST['portalURL']) ? $_POST['portalURL'] : '',
                    'configId' => $_POST['configId'],
                    'clientRedirectUri' => !empty($_POST['clientRedirectURI']) ? $_POST['clientRedirectURI'] : '',
                    'clientResourceUrl' => !empty($_POST['clientResourceURL']) ? $_POST['clientResourceURL'] : '',
                    'clientTokenUrl' => !empty($_POST['clientTokenURL']) ? $_POST['clientTokenURL'] : '',
		    'defaultCloudCredentials' => $_POST['defaultCloudCredentials']
                );
            }
	    else{
            echo json_encode(array(
                    'result' => 'noData',
                    'msg' => 'Cloud Type not supported'
                )
            );exit;
            }
            $headers = array(
                'Content-Type'      => 'application/json',
                'token'          => $this->session->get('token')
            );
            $this->log('ResultPost: '. json_encode($data), 'info');
            $result = $this->_model->saveCloudCredentials($headers, $data);
            switch ($result['status']) {
                case (200):

                    $response = array(
                        'result' => 'success',
                        'msg'    => $this->language->get('savedCloudCredentials')
                    );
                    echo json_encode($response);
                    exit;

                case (421):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('421')
                    );
                    echo json_encode($result);
                    exit();
                case (500):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('500')
                    );
                    echo json_encode($result);
                    exit();
                case (0):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('0')
                    );
                    echo json_encode($result);
                    exit();
                default:
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('default')
                    );
                    echo json_encode($result);
                    exit();
            }

        }else{
            echo json_encode(array(
                    'result' => 'noData',
                    'msg' => $this->language->get('noData')
                )
            );
        }
    }

    public function editCloudCredentials(){
        if(!empty($_POST['cloudType1'])){
	if(empty($_POST['defaultCloudCredentials1'])){
            $_POST['defaultCloudCredentials1'] = false;
            }else{
            $_POST['defaultCloudCredentials1'] = true;
            }
            /*$data = array(
                'cloudName' => $_POST['cloudType'],
                'identity' => ($_POST['cloudType'] == "localStorage") ? "local" : $_POST['identity'],
                'credential' => $_POST['accessKey'],
                'containerName' => !empty($_POST['containerName']) ? $_POST['containerName'] : '',
                'endPointUrl' => ($_POST['cloudType'] == "localStorage") ? $_POST['localStorage'] : $_POST['epaUrl'],
            );*/
            if($_POST['cloudType1'] == Language::show('azureBlob', 'settings') || $_POST['cloudType1'] == Language::show('ibmSoftlayer', 'settings') || $_POST['cloudType1'] == Language::show('amazonS3', 'settings') || $_POST['cloudType1'] == Language::show('localStorage', 'settings')){


                $data = array(

                    'cloudName' => $_POST['cloudType1'],
                    'identity' => ($_POST['cloudType1'] == "localStorage") ? "local" : $_POST['clientId1'],
                    'provider' => $_POST['cloudType1'],
                    'credential' => $_POST['access_key'],
                    'containerName' => !empty($_POST['containerName']) ? $_POST['containerName'] : '',
                    'endPointUrl' => ($_POST['cloudType1'] == "localStorage") ? $_POST['localStorage1'] : $_POST['epaUrl1'],
		    'defaultCloudCredentials' => $_POST['defaultCloudCredentials1']
                );

            }
            elseif($_POST['cloudType1'] == Language::show('googleDrive', 'settings') || $_POST['cloudType1'] == Language::show('microsoft', 'settings')){
                $data = array(
                    'cloudName' => $_POST['cloudType1'],
                    'provider' => $_POST['cloudType1'],
                    'identity' => $_POST['clientId1'],
                    'credential' => $_POST['clientSecret1'],
                    'containerName' => !empty($_POST['containerName']) ? $_POST['containerName'] : '',
                    'endPointUrl' => !empty($_POST['portalURL1']) ? $_POST['portalURL1'] : '',
                    'configId' => $_POST['configId'],
                    'clientRedirectUri' => !empty($_POST['clientRedirectURI1']) ? $_POST['clientRedirectURI1'] : '',
                    'clientResourceUrl' => !empty($_POST['clientResourceURL1']) ? $_POST['clientResourceURL1'] : '',
                    'clientTokenUrl' => !empty($_POST['clientTokenURL1']) ? $_POST['clientTokenURL1'] : '',
		    'defaultCloudCredentials' => $_POST['defaultCloudCredentials1']
                );
            }
	    else{
            echo json_encode(array(
                    'result' => 'noData',
                    'msg' => 'Cloud Type not supported'
                )
            );exit;
            }
            $headers = array(
                'Content-Type'      => 'application/json',
                'token'          => $this->session->get('token')
            );
            $this->log('ResultPOst: '. json_encode($data), 'info');
            $identity = $data['identity'];
            $result = $this->_model->updateCloudCredentials($headers, $data, $identity);
            switch ($result['status']) {
                case (200):

                    $response = array(
                        'result' => 'success',
                        'msg'    => $this->language->get('savedCloudCredentials')
                    );
                    echo json_encode($response);
                    exit;

                case (421):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('421')
                    );
                    echo json_encode($result);
                    exit();
                case (500):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('500')
                    );
                    echo json_encode($result);
                    exit();
                case (0):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('0')
                    );
                    echo json_encode($result);
                    exit();
                default:
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('default')
                    );
                    echo json_encode($result);
                    exit();
            }

        }else{
            echo json_encode(array(
                    'result' => 'noData',
                    'msg' => $this->language->get('noData')
                )
            );
        }
    }

    public function removeCloudCredentials(){
        if(!empty($_POST['identity'])){
            $headers = array(
                'Content-Type'      => 'application/json',
                'identity'          => $_POST['identity'],
                'token'             => $this->session->get('token')
            );
            $result = $this->_model->removeCloudCredentials($headers);
            $this->log("headers: ". json_encode($headers), 'info');
            switch ($result['status']) {
                case (200):

                    $response = array(
                        'result' => 'success',
                        'msg'    => $this->language->get('removeCloudCredentials')
                    );
                    echo json_encode($response);
                    exit;

                case (421):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('421')
                    );
                    echo json_encode($result);
                    exit();
                case (500):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('500')
                    );
                    echo json_encode($result);
                    exit();
                case (0):
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('0')
                    );
                    echo json_encode($result);
                    exit();
                default:
                    $result = array(
                        'result' => 'fail',
                        'msg' => $this->language->get('default')
                    );
                    echo json_encode($result);
                    exit();
            }

        }else{
            echo json_encode(array(
                    'result' => 'noData',
                    'msg' => $this->language->get('noData')
                )
            );
        }
    }

}
